const selectFormat = require('./utils/selectFormat');
const buildPaginationQuery = require('./utils/buildPaginationQuery');

const idOrFilter = id => (typeof id === 'object' ? id : {id});

module.exports = (pool) => {
    const db = {
        query: pool.query.bind(pool),
        format: pool.format,
        insert: (tableName, params) =>
            pool.query('INSERT INTO ?? SET ?', [tableName, params]).then(({insertId}) => insertId),
        replace: (tableName, params) => pool.query('REPLACE INTO ?? SET ?', [tableName, params]),
        update: (tableName, filter, params) =>
            pool
                .query(`UPDATE ?? SET ? WHERE ${selectFormat(idOrFilter(filter))}`, [tableName, params])
                .then(({changedRows}) => changedRows > 0),
        select: (tableName, filter, selector = '*', pagination, conditionsConfig) =>
            pool.query(
                `
      SELECT ${selector} 
      FROM ?? 
      WHERE ${selectFormat(idOrFilter(filter), conditionsConfig)} 
      ${pagination ? buildPaginationQuery(pagination) : ''}`,
                [tableName],
            ),
        get: (...args) => db.select(...args).then(([result]) => result),
        delete: (tableName, filter) =>
            pool
                .query(`DELETE FROM ?? WHERE ${selectFormat(idOrFilter(filter))}`, [tableName])
                .then(({affectedRows}) => affectedRows > 0),
    };
    return db;
};
